/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright (C) 2013 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Pawel Stolowski <pawel.stolowski@canonical.com>
 */

#include "SelfServiceBundlesScope.h"
#include "SelfServiceBundlesQuery.h"
#include <QCoreApplication>
#include <QStringList>
#include <QLocale>
#include <map>
#include <memory>
#include <string>
#include "Utils.h"
#include "config.h"

using namespace std;
using namespace unity::scopes;

#define EXPORT __attribute__ ((visibility ("default")))

extern "C" {

    EXPORT unity::scopes::ScopeBase* UNITY_SCOPE_CREATE_FUNCTION()
    {
        return new SelfServiceBundlesScope();
    }

    EXPORT void UNITY_SCOPE_DESTROY_FUNCTION(unity::scopes::ScopeBase* scope_base)
    {
        delete scope_base;
    }
}

SelfServiceBundlesScope::SelfServiceBundlesScope()
    : ScopeBase()
{
    bindtextdomain(GETTEXT_PACKAGE, LOCALE_DIR);
    textdomain(GETTEXT_PACKAGE);

    // determine locale for Smart Scopes queries
    m_locale = QLocale::system().name();

    int argc = 1;
    char *argv[1] = {nullptr};

    // an instance of Application is needed to avoid warnings from Qt
    m_app = new QCoreApplication(argc, argv);
}

SelfServiceBundlesScope::~SelfServiceBundlesScope()
{
    delete m_app;
}

int SelfServiceBundlesScope::start(string const&, unity::scopes::RegistryProxy const&)
{
    return VERSION;
}

SearchQueryBase::UPtr SelfServiceBundlesScope::search(const CannedQuery &query, SearchMetadata const&)
{
    QStringList scopes;
    SearchQueryBase::UPtr q(new SelfServiceBundlesQuery(query, scopes, m_locale));
    return q;
}

PreviewQueryBase::UPtr SelfServiceBundlesScope::preview(Result const& result,
                                                 ActionMetadata const& hints)
{
    PreviewQueryBase::UPtr previewer(new SelfServiceBundlesPreview(*this, result));
    return previewer;
}

void SelfServiceBundlesScope::appendIfEnabled(QStringList &scopeToQuery, const QString &scopeId) const
{
    if (!m_disabledScopes.contains(scopeId)) {
        scopeToQuery << scopeId;
    }
}

void SelfServiceBundlesScope::updateDisabledScopes()
{
}

SelfServiceBundlesPreview::SelfServiceBundlesPreview(SelfServiceBundlesScope &scope, Result const &result)
    : scope(scope),
    result(result)
{
}

void SelfServiceBundlesPreview::cancelled() {
}

void SelfServiceBundlesPreview::run(PreviewReplyProxy const &reply) {
    ColumnLayout layout(1);
    layout.add_column({"art", "actions"});
    reply->register_layout({layout});

    PreviewWidget art("art", "image");
    art.add_attribute_mapping("source", "art");

    PreviewWidget actions("actions", "actions");
    {
        VariantBuilder builder;
        builder.add_tuple({
            {"id", Variant("open")},
            {"label", Variant("Buy now")}
        });
        actions.add_attribute_value("actions", builder.end());
    }

    reply->push({art, actions});
}
